package com.factory.web.service.pm;

import java.util.List;

import com.baomidou.mybatisplus.extension.service.IService;
import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.web.entity.pm.OnsiteApprovalPlan;
import com.factory.web.entity.pm.OnsiteCalendar;
import com.factory.web.entity.pm.OnsiteItem;
import com.factory.web.entity.pm.OnsiteTitle;
import com.factory.web.entity.pm.OnsiteWorkshop;
import com.factory.web.entity.pm.req.AddItemResultListReq;
import com.factory.web.entity.pm.req.AddPmPlanReq;
import com.factory.web.entity.pm.req.EditItemResultListReq;
import com.factory.web.entity.pm.req.EditPmPlanReq;
import com.factory.web.entity.pm.req.PmPlanPauseFlagReq;
import com.factory.web.entity.pm.req.SearchAllWorkshopReq;
import com.factory.web.entity.pm.req.SearchApprovalUserReq;
import com.factory.web.entity.pm.req.SearchItemListReq;
import com.factory.web.entity.pm.req.SearchPmAllCheckFlagReq;
import com.factory.web.entity.pm.req.SearchPmCheckRecordListReq;
import com.factory.web.entity.pm.req.SearchPmItemNoGoListReq;
import com.factory.web.entity.pm.req.SearchPmPlanCheckListReq;
import com.factory.web.entity.pm.req.SearchPmPlanListForPadReq;
import com.factory.web.entity.pm.req.SearchPmPlanListReq;
import com.factory.web.entity.pm.req.SearchPmPlanReq;
import com.factory.web.entity.pm.req.SearchPmWorkshopNoGoListReq;
import com.factory.web.entity.pm.req.SearchTaskReq;
import com.factory.web.entity.pm.req.SearchWorkshopReq;
import com.factory.web.entity.pm.res.ApprovalUserRes;
import com.factory.web.entity.pm.res.PmAllCheckFlagRes;
import com.factory.web.entity.pm.res.PmAllWorkshopListRes;
import com.factory.web.entity.pm.res.PmCheckListRes;
import com.factory.web.entity.pm.res.PmCheckRecordListRes;
import com.factory.web.entity.pm.res.PmItemNoGoListRes;
import com.factory.web.entity.pm.res.PmNoGoTaskAapprovalRes;
import com.factory.web.entity.pm.res.PmPlanCheckListRes;
import com.factory.web.entity.pm.res.PmPlanListForPadRes;
import com.factory.web.entity.pm.res.PmPlanListRes;
import com.factory.web.entity.pm.res.PmPlanRes;
import com.factory.web.entity.pm.res.PmTaskListRes;
import com.factory.web.entity.pm.res.PmWorkshopNoGoListRes;

/**
 * 防错计划
 *
 * @author wzy
 * @date 2021-07-09 12:00:00
 */
public interface PreventMistakeService extends IService<BentelerPlan> {

	/**
	 * 新建防错计划
	 * 
	 * @param req
	 * @return
	 */
	public ResponseBean addPmPlan(AddPmPlanReq req);

	/**
	 * 校验计划名是否存在
	 * 
	 * @param name
	 * @return true存在，false不存在
	 */
	public boolean checkPlanNameExist(String name);

	/**
	 * 校验工作站名是否存在
	 * 
	 * @param wsName
	 * @return true存在，false不存在
	 */
	public boolean checkWsNameExist(String wsName);

	/**
	 * 取得防错计划点检列表
	 * 
	 * @param req
	 * @return 防错计划点检列表
	 */
	public ResponseBean<PageBean<PmPlanCheckListRes>> getPmPlanCheckList(SearchPmPlanCheckListReq req);

	/**
	 * 取得防错计划列表
	 * 
	 * @param req
	 * @return 防错计划列表
	 */
	public ResponseBean<PageBean<PmPlanListRes>> getPmPlanList(SearchPmPlanListReq req);

	/**
	 * 取得防错计划当天任务信息
	 * 
	 * @param req
	 * @return 防错计划当天任务信息
	 */
	public List<PmTaskListRes> getTaskList(SearchPmPlanReq req);

	/**
	 * 取得防错计划信息
	 * 
	 * @param req
	 * @return 防错计划信息
	 */
	public PmPlanRes getPmPlan(SearchPmPlanReq req);

	/**
	 * 禁用解禁防错计划
	 * 
	 * @param req
	 * @return
	 */
	public ResponseBean disablePmPlan(PmPlanPauseFlagReq req);

	/**
	 * 取得工作站信息
	 * 
	 * @param req
	 * @return 工作站信息
	 */
	public ResponseBean<OnsiteWorkshop> getWorkshop(SearchWorkshopReq req);

	/**
	 * 修改防错计划
	 * 
	 * @param req
	 * @return
	 */
	public ResponseBean editPmPlan(EditPmPlanReq req);

	/**
	 * 取得计划审批信息
	 * 
	 * @param req
	 * @return 计划审批信息
	 */
	public ResponseBean<List<OnsiteApprovalPlan>> getPmPlanApprovalList(SearchPmPlanReq req);

	/**
	 * 取得巡检规则信息
	 * 
	 * @param req
	 * @return 巡检规则信息
	 */
	public ResponseBean<List<OnsiteCalendar>> getCalendarList(SearchPmPlanReq req);

	/**
	 * 取得检查记录列表
	 * 
	 * @param req
	 * @return 检查记录列表
	 */
	public ResponseBean<List<PmCheckRecordListRes>> getPmCheckRecordList(SearchPmCheckRecordListReq req);

	/**
	 * 取得Pad端防错计划列表
	 * 
	 * @param req
	 * @return Pad端防错计划列表
	 */
	public ResponseBean<List<PmPlanListForPadRes>> getPmPlanListForPad(SearchPmPlanListForPadReq req);

	/**
	 * 取得表头信息
	 * 
	 * @param bentelerPlanId
	 * @return 表头信息
	 */
	public OnsiteTitle getTitle(Long bentelerPlanId);

	/**
	 * 取得工作站列表
	 * 
	 * @param req
	 * @return 工作站列表
	 */
	public ResponseBean<List<OnsiteWorkshop>> getWorkshopList(SearchPmAllCheckFlagReq req);

	/**
	 * 取得巡检项目列表
	 * 
	 * @param req
	 * @return 巡检项目列表
	 */
	public ResponseBean<List<OnsiteItem>> getItemList(SearchItemListReq req);

	/**
	 * 添加巡检项目检查结果
	 * 
	 * @param req
	 * @return
	 */
	public ResponseBean addItemResult(AddItemResultListReq req);

	/**
	 * 任务提交
	 * 
	 * @param req
	 * @return
	 */
	public ResponseBean submitTask(SearchTaskReq req);

	/**
	 * 取得审批人
	 * 
	 * @param req
	 * @return 审批人
	 */
	public ResponseBean<ApprovalUserRes> getApprovalUser(SearchApprovalUserReq req);
	
	/**
     * 取得审批人
     * @param approvalLevel 1是1级审批 2是2级审批
     * @param type 1计划审批  2首件任务审批  3防错任务审批
     * @return 审批人
     */
    public ApprovalUserRes getApprovalUserInfo(int approvalLevel, int type);

	/**
	 * 取得每频次点检信息
	 * 
	 * @param req
	 * @return 每频次点检信息
	 */
	public ResponseBean<List<PmCheckListRes>> getPmCheckList(SearchPmPlanReq req);
	
	/**
	 * 取得不放行任务对应的工作站
	 * 
	 * @param req
	 * @return 不放行任务对应的工作站
	 */
	public ResponseBean<List<PmWorkshopNoGoListRes>> getPmWorkshopNoGoList(SearchPmWorkshopNoGoListReq req);
	
	/**
	 * 取得不放行任务对应的点检项目
	 * 
	 * @param req
	 * @return 不放行任务对应的点检项目
	 */
	public ResponseBean<List<PmItemNoGoListRes>> getPmItemNoGoList(SearchPmItemNoGoListReq req);
	
	/**
	 * 取得不放行任务对应的质量经理审批信息
	 * 
	 * @param req
	 * @return 不放行任务对应的质量经理审批信息
	 */
	public ResponseBean<PmNoGoTaskAapprovalRes> getPmNoGoTaskAapproval(SearchTaskReq req);
	
	/**
	 * 修改巡检项目检查结果(这里只有重检走这里)
	 * 
	 * @param req
	 * @return
	 */
	public ResponseBean editItemResult(EditItemResultListReq req);
	
	/**
     * 修改防错计划
     * @param bentelerPlanId
     * @return 
     */
	public void setOnsitePlanHis(Long bentelerPlanId);
	
	/**
	 * 任务和巡检结果记录历史
	 * 
	 * @param onsiteTaskId
	 * @return
	 */
	public void setOnsiteTaskHis(Long onsiteTaskId);
	
	/**
	 * 取得所有工作站
	 * 
	 * @param req
	 * @return 所有工作站
	 */
	public ResponseBean<List<PmAllWorkshopListRes>> getAllWorkshopList(SearchAllWorkshopReq req);
	
	/**
     * 取得计划的最新审批信息
     * @param req
     * @return 计划的最新审批信息
     */
    public OnsiteApprovalPlan getLatestApprovalInfo(SearchPmPlanReq req);
    
    /**
     * 判断检验结果是否填写完成
     * @param req
     * @return 判断检验结果是否填写完成
     */
    public PmAllCheckFlagRes getPmAllCheckFlag(SearchPmAllCheckFlagReq req);

}
